@@ -297,7 +297,10 @@ sub visitStructType {
 			print $FH "#define FREE_in_",$node->{c_name}," FREE_",$node->{c_name},"\n";
 			print $FH "#define FREE_inout_",$node->{c_name}," FREE_",$node->{c_name},"\n";
 			print $FH "#define FREE_out_",$node->{c_name},"(v) {\\\n";
-			print $FH "\t\tif (NULL != (v)) FREE_",$node->{c_name},"(v);\\\n";
+			print $FH "\t\tif (NULL != (v)) {\\\n";
+			print $FH "\t\t\tFREE_",$node->{c_name},"(v);\\\n";
+			print $FH "\t\t\tCORBA_free(v);\\\n";
+			print $FH "\t}\\\n";
 			print $FH "\t}\n";
 		}
 		print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
@@ -494,7 +497,9 @@ sub visitUnionType {
 			print $FH "#define FREE_in_",$node->{c_name}," FREE_",$node->{c_name},"\n";
 			print $FH "#define FREE_inout_",$node->{c_name}," FREE_",$node->{c_name},"\n";
 			print $FH "#define FREE_out_",$node->{c_name},"(v) {\\\n";
-			print $FH "\t\tif (NULL != (v)) FREE_",$node->{c_name},"(v);\\\n";
+			print $FH "\t\tif (NULL != (v)) {\\\n";
+			print $FH "\t\t\tFREE_",$node->{c_name},"(v);\\\n";
+			print $FH "\t\t\tCORBA_free(v);\\\n";
 			print $FH "\t}\n";
 		}
 		print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
@@ -618,23 +623,31 @@ sub visitSequenceType {
 		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
 		print $FH "\t\tif (NULL != (v)->_buffer) CORBA_free((v)->_buffer);\\\n";
-		print $FH "\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
-		print $FH "\t\tif (NULL == (v)->_buffer) goto err;\\\n";
-		print $FH "\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\tGET_inout_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\tif (0 != ",$nb,") {\\\n";
+		print $FH "\t\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
+		print $FH "\t\t\tif (NULL == (v)->_buffer) goto err;\\\n";
+		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
+		print $FH "\t\t\t\tGET_inout_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t}\\\n";
+		print $FH "\t\t} else {\\\n";
+		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define GET_out_",$node->{c_name},"(ptr,v) {\\\n";
 		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
-		print $FH "\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
-		print $FH "\t\tif (NULL == (v)->_buffer) goto err;\\\n";
-		print $FH "\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\tGET_out_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\tif (0 != ",$nb,") {\\\n";
+		print $FH "\t\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
+		print $FH "\t\t\tif (NULL == (v)->_buffer) goto err;\\\n";
+		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
+		print $FH "\t\t\t\tGET_out_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t}\\\n";
+		print $FH "\t\t} else {\\\n";
+		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define ALLOC_GET_out_",$node->{c_name}," GET_out_",$node->{c_name},"\n";
@@ -642,12 +655,16 @@ sub visitSequenceType {
 		print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
 		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
-		print $FH "\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
-		print $FH "\t\tif (NULL == (v)->_buffer) goto err;\\\n";
-		print $FH "\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\tGET_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\tif (0 != (v)->_length) {\\\n";
+		print $FH "\t\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
+		print $FH "\t\t\tif (NULL == (v)->_buffer) goto err;\\\n";
+		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
+		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
+		print $FH "\t\t\t\tGET_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t}\\\n";
+		print $FH "\t\t} else {\\\n";
+		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define FREE_in_",$node->{c_name}," FREE_",$node->{c_name},"\n";
